@@ -2,19 +2,19 @@
 /*
  * GData Client
  * Copyright (C) Philip Withnall 2008-2009 <philip@tecnocode.co.uk>
- * 
- * GData Client is free software: you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation, either version 3 of the License, or
- * (at your option) any later version.
+ *
+ * GData Client is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * GData Client is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with GData Client.  If not, see <http://www.gnu.org/licenses/>.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with GData Client.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -45,20 +45,27 @@ static void gdata_entry_finalize (GObject *object);
 static void gdata_entry_get_property (GObject *object, guint property_id, GValue *value, GParamSpec *pspec);
 static void gdata_entry_set_property (GObject *object, guint property_id, const GValue *value, GParamSpec *pspec);
 static void real_get_xml (GDataEntry *self, GString *xml_string);
+static gboolean real_parse_xml (GDataEntry *self, xmlDoc *doc, xmlNode *node, GError **error);
+static void real_get_namespaces (GDataEntry *self, GHashTable *namespaces);
 
 struct _GDataEntryPrivate {
 	gchar *title;
 	gchar *id;
+	gchar *etag;
 	GTimeVal updated;
 	GTimeVal published;
 	GList *categories;
 	gchar *content;
 	GList *links;
 	GList *authors;
+
+	GString *extra_xml;
+	GHashTable *extra_namespaces;
 };
 
 enum {
 	PROP_TITLE = 1,
+	PROP_ETAG,
 	PROP_ID,
 	PROP_UPDATED,
 	PROP_PUBLISHED,
@@ -81,6 +88,8 @@ gdata_entry_class_init (GDataEntryClass *klass)
 	gobject_class->finalize = gdata_entry_finalize;
 
 	klass->get_xml = real_get_xml;
+	klass->parse_xml = real_parse_xml;
+	klass->get_namespaces = real_get_namespaces;
 
 	g_object_class_install_property (gobject_class, PROP_TITLE,
 				g_param_spec_string ("title",
@@ -91,16 +100,22 @@ gdata_entry_class_init (GDataEntryClass *klass)
 				g_param_spec_string ("id",
 					"ID", "The ID for this entry.",
 					NULL,
-					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+					G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
+	/* Since: 0.2.0 */
+	g_object_class_install_property (gobject_class, PROP_ETAG,
+				g_param_spec_string ("etag",
+					"ETag", "The ETag for this entry.",
+					NULL,
+					G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_STATIC_STRINGS));
 	g_object_class_install_property (gobject_class, PROP_UPDATED,
 				g_param_spec_boxed ("updated",
 					"Updated", "The last update time for this entry.",
-					G_TYPE_TIME_VAL,
+					GDATA_TYPE_G_TIME_VAL,
 					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 	g_object_class_install_property (gobject_class, PROP_PUBLISHED,
 				g_param_spec_boxed ("published",
 					"Published", "The time this entry was published.",
-					G_TYPE_TIME_VAL,
+					GDATA_TYPE_G_TIME_VAL,
 					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 	g_object_class_install_property (gobject_class, PROP_CONTENT,
 				g_param_spec_string ("content",
@@ -118,6 +133,8 @@ static void
 gdata_entry_init (GDataEntry *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_ENTRY, GDataEntryPrivate);
+	self->priv->extra_xml = g_string_new ("");
+	self->priv->extra_namespaces = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 }
 
 static void
@@ -127,6 +144,7 @@ gdata_entry_finalize (GObject *object)
 
 	g_free (priv->title);
 	xmlFree (priv->id);
+	xmlFree (priv->etag);
 	g_list_foreach (priv->categories, (GFunc) gdata_category_free, NULL);
 	g_list_free (priv->categories);
 	g_free (priv->content);
@@ -135,6 +153,9 @@ gdata_entry_finalize (GObject *object)
 	g_list_foreach (priv->authors, (GFunc) gdata_author_free, NULL);
 	g_list_free (priv->authors);
 
+	g_string_free (priv->extra_xml, TRUE);
+	g_hash_table_destroy (priv->extra_namespaces);
+
 	/* Chain up to the parent class */
 	G_OBJECT_CLASS (gdata_entry_parent_class)->finalize (object);
 }
@@ -151,6 +172,9 @@ gdata_entry_get_property (GObject *object, guint property_id, GValue *value, GPa
 		case PROP_ID:
 			g_value_set_string (value, priv->id);
 			break;
+		case PROP_ETAG:
+			g_value_set_string (value, priv->etag);
+			break;
 		case PROP_UPDATED:
 			g_value_set_boxed (value, &(priv->updated));
 			break;
@@ -176,6 +200,14 @@ gdata_entry_set_property (GObject *object, guint property_id, const GValue *valu
 	GDataEntry *self = GDATA_ENTRY (object);
 
 	switch (property_id) {
+		case PROP_ID:
+			/* Construct only */
+			self->priv->id = g_value_dup_string (value);
+			break;
+		case PROP_ETAG:
+			/* Construct only */
+			self->priv->etag = g_value_dup_string (value);
+			break;
 		case PROP_TITLE:
 			gdata_entry_set_title (self, g_value_get_string (value));
 			break;
@@ -244,9 +276,9 @@ real_get_xml (GDataEntry *self, GString *xml_string)
 		g_string_append_printf (xml_string, "<link href='%s'", link->href);
 
 		if (G_UNLIKELY (link->title != NULL)) {
-			gchar *title = g_markup_escape_text (link->title, -1);
-			g_string_append_printf (xml_string, " title='%s'", title);
-			g_free (title);
+			gchar *link_title = g_markup_escape_text (link->title, -1);
+			g_string_append_printf (xml_string, " title='%s'", link_title);
+			g_free (link_title);
 		}
 
 		if (G_LIKELY (link->rel != NULL))
@@ -281,37 +313,33 @@ real_get_xml (GDataEntry *self, GString *xml_string)
 
 		g_string_append (xml_string, "</author>");
 	}
+
+	if (priv->extra_xml != NULL && priv->extra_xml->str != NULL)
+		g_string_append (xml_string, priv->extra_xml->str);
+}
+
+static void
+real_get_namespaces (GDataEntry *self, GHashTable *namespaces)
+{
+	g_hash_table_insert (namespaces, (gchar*) "gd", (gchar*) "http://schemas.google.com/g/2005");
 }
 
 /**
  * gdata_entry_new:
+ * @id: the entry's ID, or %NULL
  *
- * Creates a new #GDataEntry with default properties.
+ * Creates a new #GDataEntry with the given ID and default properties.
  *
- * Return value: a new #GDataEntry
+ * Return value: a new #GDataEntry; unref with g_object_unref()
  **/
 GDataEntry *
-gdata_entry_new (void)
+gdata_entry_new (const gchar *id)
 {
-	return g_object_new (GDATA_TYPE_ENTRY, NULL);
+	return g_object_new (GDATA_TYPE_ENTRY, "id", id, NULL);
 }
 
-/**
- * gdata_entry_new_from_xml:
- * @xml: the XML for just the entry, with full namespace declarations
- * @length: the length of @xml, or -1
- * @error: a #GError, or %NULL
- *
- * Creates a new #GDataEntry from the provided @xml.
- *
- * If @length is -1, @xml will be assumed to be nul-terminated.
- *
- * If an error occurs during parsing, a suitable error from #GDataParserError will be returned.
- *
- * Return value: a new #GDataEntry, or %NULL
- **/
 GDataEntry *
-gdata_entry_new_from_xml (const gchar *xml, gint length, GError **error)
+_gdata_entry_new_from_xml (GType entry_type, const gchar *xml, gint length, GError **error)
 {
 	/* TODO: Shouldn't this be private? */
 	xmlDoc *doc;
@@ -350,23 +378,52 @@ gdata_entry_new_from_xml (const gchar *xml, gint length, GError **error)
 		return NULL;
 	}
 
-	return _gdata_entry_new_from_xml_node (doc, node, error);
+	return _gdata_entry_new_from_xml_node (entry_type, doc, node, error);
+}
+
+/**
+ * gdata_entry_new_from_xml:
+ * @xml: the XML for just the entry, with full namespace declarations
+ * @length: the length of @xml, or -1
+ * @error: a #GError, or %NULL
+ *
+ * Creates a new #GDataEntry from the provided @xml.
+ *
+ * If @length is -1, @xml will be assumed to be nul-terminated.
+ *
+ * If an error occurs during parsing, a suitable error from #GDataParserError will be returned.
+ *
+ * Return value: a new #GDataEntry, or %NULL
+ **/
+GDataEntry *
+gdata_entry_new_from_xml (const gchar *xml, gint length, GError **error)
+{
+	return _gdata_entry_new_from_xml (GDATA_TYPE_ENTRY, xml, length, error);
 }
 
 GDataEntry *
-_gdata_entry_new_from_xml_node (xmlDoc *doc, xmlNode *node, GError **error)
+_gdata_entry_new_from_xml_node (GType entry_type, xmlDoc *doc, xmlNode *node, GError **error)
 {
 	GDataEntry *entry;
+	GDataEntryClass *klass;
 
+	g_return_val_if_fail (g_type_is_a (entry_type, GDATA_TYPE_ENTRY) == TRUE, FALSE);
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (node != NULL, FALSE);
 	g_return_val_if_fail (xmlStrcmp (node->name, (xmlChar*) "entry") == 0, FALSE);
 
-	entry = gdata_entry_new ();
+	entry = g_object_new (entry_type, NULL);
+
+	klass = GDATA_ENTRY_GET_CLASS (entry);
+	if (klass->parse_xml == NULL)
+		return FALSE;
+
+	/* Get the ETag first */
+	entry->priv->etag = (gchar*) xmlGetProp (node, (xmlChar*) "etag");
 
 	node = node->xmlChildrenNode;
 	while (node != NULL) {
-		if (_gdata_entry_parse_xml_node (entry, doc, node, error) == FALSE) {
+		if (klass->parse_xml (entry, doc, node, error) == FALSE) {
 			g_object_unref (entry);
 			return NULL;
 		}
@@ -376,8 +433,8 @@ _gdata_entry_new_from_xml_node (xmlDoc *doc, xmlNode *node, GError **error)
 	return entry;
 }
 
-gboolean
-_gdata_entry_parse_xml_node (GDataEntry *self, xmlDoc *doc, xmlNode *node, GError **error)
+static gboolean
+real_parse_xml (GDataEntry *self, xmlDoc *doc, xmlNode *node, GError **error)
 {
 	g_return_val_if_fail (GDATA_IS_ENTRY (self), FALSE);
 	g_return_val_if_fail (doc != NULL, FALSE);
@@ -498,7 +555,26 @@ _gdata_entry_parse_xml_node (GDataEntry *self, xmlDoc *doc, xmlNode *node, GErro
 		xmlFree (uri);
 		xmlFree (email);
 	} else {
-		return gdata_parser_error_unhandled_element ((gchar*) node->ns->prefix, (gchar*) node->name, "entry", error);
+		xmlBuffer *buffer;
+		xmlNs **namespaces, **namespace;
+
+		/* Unhandled XML */
+		buffer = xmlBufferCreate ();
+		xmlNodeDump (buffer, doc, node, 0, 0);
+		g_string_append (self->priv->extra_xml, (gchar*) xmlBufferContent (buffer));
+		g_message ("Unhandled XML: %s", (gchar*) xmlBufferContent (buffer));
+		xmlBufferFree (buffer);
+
+		/* Get the namespaces */
+		namespaces = xmlGetNsList (doc, node);
+		for (namespace = namespaces; *namespace != NULL; namespace++) {
+			if ((*namespace)->prefix != NULL) {
+				g_hash_table_insert (self->priv->extra_namespaces,
+						     g_strdup ((gchar*) ((*namespace)->prefix)),
+						     g_strdup ((gchar*) ((*namespace)->href)));
+			}
+		}
+		xmlFree (namespaces);
 	}
 
 	return TRUE;
@@ -552,6 +628,24 @@ gdata_entry_get_id (GDataEntry *self)
 	return self->priv->id;
 }
 
+/**
+ * gdata_entry_get_etag:
+ * @self: a #GDataEntry
+ *
+ * Returns the ETag of the entry; a unique identifier for each version of the entry. For more information, see the
+ * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/reference.html#ResourceVersioning">online documentation</ulink>.
+ *
+ * Return value: the entry's ETag
+ *
+ * Since: 0.2.0
+ **/
+const gchar *
+gdata_entry_get_etag (GDataEntry *self)
+{
+	g_return_val_if_fail (GDATA_IS_ENTRY (self), NULL);
+	return self->priv->etag;
+}
+
 /**
  * gdata_entry_get_updated:
  * @self: a #GDataEntry
@@ -592,13 +686,29 @@ gdata_entry_get_published (GDataEntry *self, GTimeVal *published)
 void
 gdata_entry_add_category (GDataEntry *self, GDataCategory *category)
 {
-	/* TODO: More category API */
 	g_return_if_fail (GDATA_IS_ENTRY (self));
 	g_return_if_fail (category != NULL);
 
 	self->priv->categories = g_list_prepend (self->priv->categories, category);
 }
 
+/**
+ * gdata_entry_get_categories:
+ * @self: a #GDataEntry
+ *
+ * Gets a list of the #GDataCategory<!-- -->s containing this entry.
+ *
+ * Return value: a #GList of #GDataCategory<!-- -->s
+ *
+ * Since: 0.2.0
+ **/
+GList *
+gdata_entry_get_categories (GDataEntry *self)
+{
+	g_return_val_if_fail (GDATA_IS_ENTRY (self), NULL);
+	return self->priv->categories;
+}
+
 /**
  * gdata_entry_get_content:
  * @self: a #GDataEntry
@@ -655,16 +765,18 @@ link_compare_cb (const GDataLink *link, const gchar *rel)
 }
 
 /**
- * gdata_entry_lookup_link:
+ * gdata_entry_look_up_link:
  * @self: a #GDataEntry
  * @rel: the value of the <structfield>rel</structfield> attribute of the desired link
  *
  * Looks up a link by <structfield>rel</structfield> value from the list of links in the entry.
  *
  * Return value: a #GDataLink, or %NULL if one was not found
+ *
+ * Since: 0.1.1
  **/
 GDataLink *
-gdata_entry_lookup_link (GDataEntry *self, const gchar *rel)
+gdata_entry_look_up_link (GDataEntry *self, const gchar *rel)
 {
 	GList *element;
 
@@ -714,6 +826,12 @@ gdata_entry_is_inserted (GDataEntry *self)
 	return FALSE;
 }
 
+static void
+build_namespaces_cb (gchar *prefix, gchar *href, GString *output)
+{
+	g_string_append_printf (output, " xmlns:%s='%s'", prefix, href);
+}
+
 /**
  * gdata_entry_get_xml:
  * @self: a #GDataEntry
@@ -729,21 +847,30 @@ gdata_entry_get_xml (GDataEntry *self)
 {
 	GDataEntryClass *klass;
 	GString *xml_string;
-	const gchar *namespaces;
+	GHashTable *namespaces;
 
 	klass = GDATA_ENTRY_GET_CLASS (self);
-	if (klass->get_xml == NULL)
-		return NULL;
+	g_assert (klass->get_xml != NULL);
+	g_assert (klass->get_namespaces != NULL);
 
 	/* Get the namespaces the class uses */
-	if (klass->get_namespaces == NULL)
-		namespaces = "";
-	else
-		namespaces = klass->get_namespaces (self);
+	namespaces = g_hash_table_new (g_str_hash, g_str_equal);
+	klass->get_namespaces (self, namespaces);
 
-	/* Allocate enough space for 300 characters, which is a decent average entry length */
-	xml_string = g_string_sized_new (300);
-	g_string_append_printf (xml_string, "<entry xmlns='http://www.w3.org/2005/Atom' %s>", namespaces);
+	/* Build up the namespace list */
+	xml_string = g_string_new ("<entry xmlns='http://www.w3.org/2005/Atom'");
+	g_hash_table_foreach (namespaces, (GHFunc) build_namespaces_cb, xml_string);
+	g_hash_table_foreach (self->priv->extra_namespaces, (GHFunc) build_namespaces_cb, xml_string);
+
+	/* Add the entry's ETag, if available */
+	if (self->priv->etag != NULL)
+		g_string_append_printf (xml_string, " gd:etag='%s'>", self->priv->etag);
+	else
+		g_string_append_c (xml_string, '>');
+
+	g_hash_table_destroy (namespaces);
+
+	/* Add the rest of the XML */
 	klass->get_xml (self, xml_string);
 	g_string_append (xml_string, "</entry>");
 
