@@ -37,6 +37,10 @@
 #include "gdata-service.h"
 #include "gdata-private.h"
 #include "gdata-marshal.h"
+#include "gdata-types.h"
+
+/* The default e-mail domain to use for usernames */
+#define EMAIL_DOMAIN "gmail.com"
 
 GQuark
 gdata_service_error_quark (void)
@@ -58,6 +62,7 @@ static gboolean real_parse_authentication_response (GDataService *self, guint st
 static void real_append_query_headers (GDataService *self, SoupMessage *message);
 static void real_parse_error_response (GDataService *self, guint status, const gchar *reason_phrase,
 				       const gchar *response_body, gint length, GError **error);
+static void notify_proxy_uri_cb (GObject *gobject, GParamSpec *pspec, GObject *self);
 
 struct _GDataServicePrivate {
 	SoupSession *session;
@@ -73,7 +78,8 @@ enum {
 	PROP_CLIENT_ID = 1,
 	PROP_USERNAME,
 	PROP_PASSWORD,
-	PROP_AUTHENTICATED
+	PROP_AUTHENTICATED,
+	PROP_PROXY_URI
 };
 
 enum {
@@ -120,7 +126,7 @@ gdata_service_class_init (GDataServiceClass *klass)
 	/**
 	 * GDataService:username:
 	 *
-	 * The user's Google or YouTube username for authentication.
+	 * The user's Google or YouTube username for authentication. This will always be a full e-mail address.
 	 **/
 	g_object_class_install_property (gobject_class, PROP_USERNAME,
 				g_param_spec_string ("username",
@@ -150,6 +156,19 @@ gdata_service_class_init (GDataServiceClass *klass)
 					FALSE,
 					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
+	/**
+	 * GDataService:proxy-uri:
+	 *
+	 * The proxy URI used internally for all Internet requests.
+	 *
+	 * Since: 0.2.0
+	 **/
+	g_object_class_install_property (gobject_class, PROP_PROXY_URI,
+				g_param_spec_boxed ("proxy-uri",
+					"Proxy URI", "The proxy URI used internally for all Internet requests.",
+					SOUP_TYPE_URI,
+					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
 	/**
 	 * GDataService::captcha-challenge:
 	 * @service: the #GDataService which received the challenge
@@ -175,6 +194,9 @@ gdata_service_init (GDataService *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_SERVICE, GDataServicePrivate);
 	self->priv->session = soup_session_sync_new ();
+
+	/* Proxy the SoupSession's proxy-uri property */
+	g_signal_connect (self->priv->session, "notify::proxy-uri", (GCallback) notify_proxy_uri_cb, self);
 }
 
 static void
@@ -222,6 +244,9 @@ gdata_service_get_property (GObject *object, guint property_id, GValue *value, G
 		case PROP_AUTHENTICATED:
 			g_value_set_boolean (value, priv->authenticated);
 			break;
+		case PROP_PROXY_URI:
+			g_value_set_boxed (value, gdata_service_get_proxy_uri (GDATA_SERVICE (object)));
+			break;
 		default:
 			/* We don't have any other property... */
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -238,6 +263,9 @@ gdata_service_set_property (GObject *object, guint property_id, const GValue *va
 		case PROP_CLIENT_ID:
 			priv->client_id = g_value_dup_string (value);
 			break;
+		case PROP_PROXY_URI:
+			gdata_service_set_proxy_uri (GDATA_SERVICE (object), g_value_get_boxed (value));
+			break;
 		default:
 			/* We don't have any other property... */
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -323,7 +351,12 @@ set_authentication_details_cb (AuthenticateAsyncData *data)
 	GDataServicePrivate *priv = data->service->priv;
 
 	g_free (priv->username);
-	priv->username = g_strdup (data->username);
+	/* Ensure the username is always a full e-mail address */
+	if (strchr (data->username, '@') == NULL)
+		priv->username = g_strdup_printf ("%s@" EMAIL_DOMAIN, data->username);
+	else
+		priv->username = g_strdup (data->username);
+
 	g_free (priv->password);
 	priv->password = g_strdup (data->password);
 	priv->authenticated = TRUE;
@@ -334,6 +367,8 @@ set_authentication_details_cb (AuthenticateAsyncData *data)
 	g_object_notify (service, "authenticated");
 	g_object_thaw_notify (service);
 
+	authenticate_async_data_free (data);
+
 	return FALSE;
 }
 
@@ -347,6 +382,7 @@ authenticate_thread (GSimpleAsyncResult *result, GDataService *service, GCancell
 	if (g_cancellable_set_error_if_cancelled (cancellable, &error) == TRUE) {
 		g_simple_async_result_set_from_error (result, error);
 		g_error_free (error);
+		authenticate_async_data_free (data);
 		return;
 	}
 
@@ -355,11 +391,11 @@ authenticate_thread (GSimpleAsyncResult *result, GDataService *service, GCancell
 	if (data->success == FALSE) {
 		g_simple_async_result_set_from_error (result, error);
 		g_error_free (error);
+		authenticate_async_data_free (data);
+		return;
 	}
 
-	/* Update the authentication details held by the service.
-	 * This should always be executed before the callback in g_simple_async_result_run_in_thread ---
-	 * if not, data will already have been freed, and crashage will happen. */
+	/* Update the authentication details held by the service */
 	data->service = service;
 	g_idle_add ((GSourceFunc) set_authentication_details_cb, data);
 }
@@ -397,7 +433,7 @@ gdata_service_authenticate_async (GDataService *self, const gchar *username, con
 	data->password = g_strdup (password);
 
 	result = g_simple_async_result_new (G_OBJECT (self), callback, user_data, gdata_service_authenticate_async);
-	g_simple_async_result_set_op_res_gpointer (result, data, (GDestroyNotify) authenticate_async_data_free);
+	g_simple_async_result_set_op_res_gpointer (result, data, NULL);
 	g_simple_async_result_run_in_thread (result, (GSimpleAsyncThreadFunc) authenticate_thread, G_PRIORITY_DEFAULT, cancellable);
 	g_object_unref (result);
 }
@@ -492,7 +528,7 @@ authenticate (GDataService *self, const gchar *username, const gchar *password,
 
 		if (strncmp (error_start, "CaptchaRequired", error_end - error_start) == 0) {
 			const gchar *captcha_base_uri = "http://www.google.com/accounts/";
-			gchar *captcha_start, *captcha_end, *captcha_uri, *captcha_answer;
+			gchar *captcha_start, *captcha_end, *captcha_uri, *new_captcha_answer;
 			guint captcha_base_uri_length;
 
 			/* CAPTCHA required to log in */
@@ -513,10 +549,10 @@ authenticate (GDataService *self, const gchar *username, const gchar *password,
 			captcha_uri[captcha_base_uri_length + (captcha_end - captcha_start)] = '\0';
 
 			/* Request a CAPTCHA answer from the application */
-			g_signal_emit (self, service_signals[SIGNAL_CAPTCHA_CHALLENGE], 0, captcha_uri, &captcha_answer);
+			g_signal_emit (self, service_signals[SIGNAL_CAPTCHA_CHALLENGE], 0, captcha_uri, &new_captcha_answer);
 			g_free (captcha_uri);
 
-			if (captcha_answer == NULL || *captcha_answer == '\0') {
+			if (new_captcha_answer == NULL || *new_captcha_answer == '\0') {
 				g_set_error_literal (error, GDATA_AUTHENTICATION_ERROR, GDATA_AUTHENTICATION_ERROR_CAPTCHA_REQUIRED,
 						     _("A CAPTCHA must be filled out to log in."));
 				goto login_error;
@@ -535,7 +571,7 @@ authenticate (GDataService *self, const gchar *username, const gchar *password,
 			/* Save the CAPTCHA token and answer, and attempt to log in with them */
 			g_object_unref (message);
 
-			return authenticate (self, username, password, g_strndup (captcha_start, captcha_end - captcha_start), captcha_answer,
+			return authenticate (self, username, password, g_strndup (captcha_start, captcha_end - captcha_start), new_captcha_answer,
 					     cancellable, error);
 		} else if (strncmp (error_start, "Unknown", error_end - error_start) == 0) {
 			goto protocol_error; /* TODO: is this really a protocol error? It's an error with *our* code */
@@ -603,7 +639,13 @@ login_error:
 	if (retval == TRUE) {
 		/* Update several properties the service holds */
 		g_free (priv->username);
-		priv->username = g_strdup (username);
+
+		/* Ensure the username is always a full e-mail address */
+		if (strchr (username, '@') == NULL)
+			priv->username = g_strdup_printf ("%s@" EMAIL_DOMAIN, username);
+		else
+			priv->username = g_strdup (username);
+
 		g_free (priv->password);
 		priv->password = g_strdup (password);
 
@@ -637,7 +679,8 @@ general_error:
  * @error: a #GError, or %NULL
  *
  * Authenticates the #GDataService with the online service using @username and @password; i.e. logs into the service with the given
- * user account.
+ * user account. @username should be a full e-mail address (e.g. <literal>john.smith@gmail.com</literal>). If a full e-mail address is
+ * not given, @username will have <literal>@gmail.com</literal> appended to create an e-mail address
  *
  * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
  * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
@@ -661,11 +704,50 @@ gdata_service_authenticate (GDataService *self, const gchar *username, const gch
 	return authenticate (self, username, password, NULL, NULL, cancellable, error);
 }
 
+guint
+_gdata_service_send_message (GDataService *self, SoupMessage *message, GError **error)
+{
+	/* Based on code from evolution-data-server's libgdata:
+	 *  Ebby Wiselyn <ebbywiselyn@gmail.com>
+	 *  Jason Willis <zenbrother@gmail.com>
+	 *
+	 * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
+	 */
+
+	soup_message_set_flags (message, SOUP_MESSAGE_NO_REDIRECT);
+	soup_session_send_message (self->priv->session, message);
+	soup_message_set_flags (message, 0);
+
+	if (SOUP_STATUS_IS_REDIRECTION (message->status_code)) {
+		SoupURI *new_uri;
+		const gchar *new_location;
+
+		new_location = soup_message_headers_get (message->response_headers, "Location");
+		g_return_val_if_fail (new_location != NULL, SOUP_STATUS_NONE);
+
+		new_uri = soup_uri_new_with_base (soup_message_get_uri (message), new_location);
+		if (new_uri == NULL) {
+			gchar *uri_string = soup_uri_to_string (new_uri, FALSE);
+			g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_PROTOCOL_ERROR,
+				     _("Invalid redirect URI: %s"), uri_string);
+			g_free (uri_string);
+			return SOUP_STATUS_NONE;
+		}
+
+		soup_message_set_uri (message, new_uri);
+		soup_uri_free (new_uri);
+
+		soup_session_send_message (self->priv->session, message);
+	}
+
+	return message->status_code;
+}
+
 typedef struct {
 	/* Input */
 	gchar *feed_uri;
 	GDataQuery *query;
-	GDataEntryParserFunc parser_func;
+	GType entry_type;
 
 	/* Output */
 	GDataFeed *feed;
@@ -699,7 +781,7 @@ query_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *c
 	}
 
 	/* Execute the query and return */
-	data->feed = gdata_service_query (service, data->feed_uri, data->query, data->parser_func, cancellable,
+	data->feed = gdata_service_query (service, data->feed_uri, data->query, data->entry_type, cancellable,
 					  data->progress_callback, data->progress_user_data, &error);
 	if (data->feed == NULL) {
 		g_simple_async_result_set_from_error (result, error);
@@ -712,7 +794,7 @@ query_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *c
  * @self: a #GDataService
  * @feed_uri: the feed URI to query, including the host name and protocol
  * @query: a #GDataQuery with the query parameters, or %NULL
- * @parser_func: a #GDataEntryParserFunc to build a #GDataEntry from the XML
+ * @entry_type: a #GType for the #GDataEntry<!-- -->s to build from the XML
  * @cancellable: optional #GCancellable object, or %NULL
  * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: data to pass to the @progress_callback function
@@ -728,7 +810,7 @@ query_thread (GSimpleAsyncResult *result, GDataService *service, GCancellable *c
  * to get the results of the operation.
  **/
 void
-gdata_service_query_async (GDataService *self, const gchar *feed_uri, GDataQuery *query, GDataEntryParserFunc parser_func, GCancellable *cancellable,
+gdata_service_query_async (GDataService *self, const gchar *feed_uri, GDataQuery *query, GType entry_type, GCancellable *cancellable,
 			   GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
 			   GAsyncReadyCallback callback, gpointer user_data)
 {
@@ -737,13 +819,12 @@ gdata_service_query_async (GDataService *self, const gchar *feed_uri, GDataQuery
 
 	g_return_if_fail (GDATA_IS_SERVICE (self));
 	g_return_if_fail (feed_uri != NULL);
-	g_return_if_fail (GDATA_IS_QUERY (query));
-	g_return_if_fail (parser_func != NULL);
+	g_return_if_fail (entry_type != G_TYPE_INVALID);
 
 	data = g_slice_new (QueryAsyncData);
 	data->feed_uri = g_strdup (feed_uri);
-	data->query = g_object_ref (query);
-	data->parser_func = parser_func;
+	data->query = (query != NULL) ? g_object_ref (query) : NULL;
+	data->entry_type = entry_type;
 	data->progress_callback = progress_callback;
 	data->progress_user_data = progress_user_data;
 
@@ -789,7 +870,7 @@ gdata_service_query_finish (GDataService *self, GAsyncResult *async_result, GErr
  * @self: a #GDataService
  * @feed_uri: the feed URI to query, including the host name and protocol
  * @query: a #GDataQuery with the query parameters, or %NULL
- * @parser_func: a #GDataEntryParserFunc to build a #GDataEntry from the XML
+ * @entry_type: a #GType for the #GDataEntry<!-- -->s to build from the XML
  * @cancellable: optional #GCancellable object, or %NULL
  * @progress_callback: a #GDataQueryProgressCallback to call when an entry is loaded, or %NULL
  * @progress_user_data: data to pass to the @progress_callback function
@@ -809,10 +890,13 @@ gdata_service_query_finish (GDataService *self, GAsyncResult *async_result, GErr
  * If the query is successful and the feed supports pagination, @query will be updated with the pagination URIs, and the next or previous page
  * can then be loaded by calling gdata_query_next_page() or gdata_query_previous_page() before running the query again.
  *
+ * If the #GDataQuery's ETag is set and it finds a match on the server, %FALSE will be returned, but @error will remain unset. Otherwise,
+ * @query's ETag will be updated with the ETag from the returned feed, if available.
+ *
  * Return value: a #GDataFeed of query results; unref with g_object_unref()
  **/
 GDataFeed *
-gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *query, GDataEntryParserFunc parser_func,
+gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *query, GType entry_type,
 		     GCancellable *cancellable, GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error)
 {
 	GDataServiceClass *klass;
@@ -837,29 +921,41 @@ gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *quer
 	if (klass->append_query_headers != NULL)
 		klass->append_query_headers (self, message);
 
+	/* Append the ETag header if possible */
+	if (query != NULL && gdata_query_get_etag (query) != NULL)
+		soup_message_headers_append (message->request_headers, "If-None-Match", gdata_query_get_etag (query));
+
 	/* Send the message */
 	status = soup_session_send_message (self->priv->session, message);
 
 	/* Check for cancellation */
 	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
 		g_object_unref (message);
-		return FALSE;
+		return NULL;
 	}
 
-	if (status != 200) {
+	if (status == 304) {
+		/* Not modified; ETag has worked */
+		g_object_unref (message);
+		return NULL;
+	} else if (status != 200) {
 		/* Error */
 		g_assert (klass->parse_error_response != NULL);
 		klass->parse_error_response (self, status, message->reason_phrase, message->response_body->data, message->response_body->length, error);
 		g_object_unref (message);
-		return FALSE;
+		return NULL;
 	}
 
 	g_assert (message->response_body->data != NULL);
 
-	feed = _gdata_feed_new_from_xml (message->response_body->data, message->response_body->length, parser_func,
+	feed = _gdata_feed_new_from_xml (message->response_body->data, message->response_body->length, entry_type,
 					 progress_callback, progress_user_data, error);
 	g_object_unref (message);
 
+	/* Update the query with the feed's ETag */
+	if (query != NULL && gdata_feed_get_etag (feed) != NULL)
+		gdata_query_set_etag (query, gdata_feed_get_etag (feed));
+
 	/* Update the query with the next and previous URIs from the feed */
 	if (query != NULL) {
 		link = gdata_feed_look_up_link (feed, "next");
@@ -877,14 +973,16 @@ gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *quer
  * gdata_service_insert_entry:
  * @self: a #GDataService
  * @upload_uri: the URI to which the upload should be sent
- * @entry: the #GDataEntry to upload
+ * @entry: the #GDataEntry to insert
  * @cancellable: optional #GCancellable object, or %NULL
  * @error: a #GError, or %NULL
  *
  * Inserts @entry by uploading it to the online service at @upload_uri. For more information about the concept of inserting entries, see
- * the <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/reference.html#Queries">online documentation</ulink> for the GData
+ * the <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/basics.html#InsertingEntry">online documentation</ulink> for the GData
  * protocol.
  *
+ * The service will return an updated version of the entry, which is the return value of this function on success.
+ *
  * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
  * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
  *
@@ -894,24 +992,25 @@ gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *quer
  * If there is an error inserting the entry, a %GDATA_SERVICE_ERROR_WITH_INSERTION error will be returned. Currently, subclasses
  * <emphasis>cannot</emphasis> cannot override this or provide more specific errors.
  *
- * Return value: %TRUE on success, %FALSE otherwise
+ * Return value: an updated #GDataEntry, or %NULL
  **/
-gboolean
+GDataEntry *
 gdata_service_insert_entry (GDataService *self, const gchar *upload_uri, GDataEntry *entry, GCancellable *cancellable, GError **error)
 {
 	GDataServiceClass *klass;
+	GDataEntry *updated_entry;
 	SoupMessage *message;
 	gchar *upload_data;
 	guint status;
 
-	g_return_val_if_fail (GDATA_IS_SERVICE (self), FALSE);
-	g_return_val_if_fail (upload_uri != NULL, FALSE);
-	g_return_val_if_fail (GDATA_IS_ENTRY (entry), FALSE);
+	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
+	g_return_val_if_fail (upload_uri != NULL, NULL);
+	g_return_val_if_fail (GDATA_IS_ENTRY (entry), NULL);
 
 	if (gdata_entry_is_inserted (entry) == TRUE) {
 		g_set_error_literal (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED,
 				     _("The entry has already been inserted."));
-		return FALSE;
+		return NULL;
 	}
 
 	message = soup_message_new (SOUP_METHOD_POST, upload_uri);
@@ -926,12 +1025,16 @@ gdata_service_insert_entry (GDataService *self, const gchar *upload_uri, GDataEn
 	soup_message_set_request (message, "application/atom+xml", SOUP_MEMORY_TAKE, upload_data, strlen (upload_data));
 
 	/* Send the message */
-	status = soup_session_send_message (self->priv->session, message);
+	status = _gdata_service_send_message (self, message, error);
+	if (status == SOUP_STATUS_NONE) {
+		g_object_unref (message);
+		return NULL;
+	}
 
 	/* Check for cancellation */
 	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
 		g_object_unref (message);
-		return FALSE;
+		return NULL;
 	}
 
 	if (status != 201) {
@@ -940,15 +1043,219 @@ gdata_service_insert_entry (GDataService *self, const gchar *upload_uri, GDataEn
 		g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_INSERTION,
 			     _("TODO: error code %u when inserting"), status);
 		g_object_unref (message);
+		return NULL;
+	}
+
+	/* Build the updated entry */
+	g_assert (message->response_body->data != NULL);
+
+	/* Parse the XML; create and return a new GDataEntry of the same type as @entry */
+	updated_entry = _gdata_entry_new_from_xml (G_OBJECT_TYPE (entry), message->response_body->data, message->response_body->length, error);
+	g_object_unref (message);
+
+	return updated_entry;
+}
+
+/**
+ * gdata_service_update_entry:
+ * @self: a #GDataService
+ * @entry: the #GDataEntry to update
+ * @cancellable: optional #GCancellable object, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Updates @entry by PUTting it to its <literal>edit</literal> link's URI. For more information about the concept of updating entries, see
+ * the <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/basics.html#UpdatingEntry">online documentation</ulink> for the GData
+ * protocol.
+ *
+ * The service will return an updated version of the entry, which is the return value of this function on success.
+ *
+ * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
+ * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
+ *
+ * If there is an error updating the entry, a %GDATA_SERVICE_ERROR_WITH_UPDATE error will be returned. Currently, subclasses
+ * <emphasis>cannot</emphasis> cannot override this or provide more specific errors.
+ *
+ * Return value: an updated #GDataEntry, or %NULL
+ *
+ * Since: 0.2.0
+ **/
+GDataEntry *
+gdata_service_update_entry (GDataService *self, GDataEntry *entry, GCancellable *cancellable, GError **error)
+{
+	GDataServiceClass *klass;
+	GDataEntry *updated_entry;
+	GDataLink *link;
+	SoupMessage *message;
+	gchar *upload_data;
+	guint status;
+
+	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
+	g_return_val_if_fail (GDATA_IS_ENTRY (entry), NULL);
+
+	/* Get the edit URI */
+	link = gdata_entry_look_up_link (entry, "edit");
+	g_assert (link != NULL);
+	message = soup_message_new (SOUP_METHOD_PUT, link->href);
+
+	/* Make sure subclasses set their headers */
+	klass = GDATA_SERVICE_GET_CLASS (self);
+	if (klass->append_query_headers != NULL)
+		klass->append_query_headers (self, message);
+
+	/* Append the ETag header if possible */
+	if (gdata_entry_get_etag (entry) != NULL)
+		soup_message_headers_append (message->request_headers, "If-Match", gdata_entry_get_etag (entry));
+
+	/* Append the data */
+	upload_data = gdata_entry_get_xml (entry);
+	soup_message_set_request (message, "application/atom+xml", SOUP_MEMORY_TAKE, upload_data, strlen (upload_data));
+
+	/* Send the message */
+	status = _gdata_service_send_message (self, message, error);
+	if (status == SOUP_STATUS_NONE) {
+		g_object_unref (message);
+		return NULL;
+	}
+
+	/* Check for cancellation */
+	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
+		g_object_unref (message);
+		return NULL;
+	}
+
+	if (status != 200) {
+		/* Error */
+		/* TODO: Handle errors more specifically, making sure to set authenticated where appropriate */
+		g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_UPDATE,
+			     _("TODO: error code %u when updating"), status);
+		g_object_unref (message);
+		return NULL;
+	}
+
+	/* Build the updated entry */
+	g_assert (message->response_body->data != NULL);
+
+	/* Parse the XML; create and return a new GDataEntry of the same type as @entry */
+	updated_entry = _gdata_entry_new_from_xml (G_OBJECT_TYPE (entry), message->response_body->data, message->response_body->length, error);
+	g_object_unref (message);
+
+	return updated_entry;
+}
+
+/**
+ * gdata_service_delete_entry:
+ * @self: a #GDataService
+ * @entry: the #GDataEntry to delete
+ * @cancellable: optional #GCancellable object, or %NULL
+ * @error: a #GError, or %NULL
+ *
+ * Deletes @entry from the server. For more information about the concept of updating entries, see the
+ * <ulink type="http" url="http://code.google.com/apis/gdata/docs/2.0/basics.html#DeletingEntry">online documentation</ulink> for the GData
+ * protocol.
+ *
+ * If @cancellable is not %NULL, then the operation can be cancelled by triggering the @cancellable object from another thread.
+ * If the operation was cancelled, the error %G_IO_ERROR_CANCELLED will be returned.
+ *
+ * If there is an error deleting the entry, a %GDATA_SERVICE_ERROR_WITH_DELETION error will be returned. Currently, subclasses
+ * <emphasis>cannot</emphasis> cannot override this or provide more specific errors.
+ *
+ * Return value: %TRUE on success, %FALSE otherwise
+ *
+ * Since: 0.2.0
+ **/
+gboolean
+gdata_service_delete_entry (GDataService *self, GDataEntry *entry, GCancellable *cancellable, GError **error)
+{
+	GDataServiceClass *klass;
+	GDataLink *link;
+	SoupMessage *message;
+	guint status;
+
+	g_return_val_if_fail (GDATA_IS_SERVICE (self), FALSE);
+	g_return_val_if_fail (GDATA_IS_ENTRY (entry), FALSE);
+
+	/* Get the edit URI */
+	link = gdata_entry_look_up_link (entry, "edit");
+	g_assert (link != NULL);
+	message = soup_message_new (SOUP_METHOD_DELETE, link->href);
+
+	/* Make sure subclasses set their headers */
+	klass = GDATA_SERVICE_GET_CLASS (self);
+	if (klass->append_query_headers != NULL)
+		klass->append_query_headers (self, message);
+
+	/* Append the ETag header if possible */
+	if (gdata_entry_get_etag (entry) != NULL)
+		soup_message_headers_append (message->request_headers, "If-Match", gdata_entry_get_etag (entry));
+
+	/* Send the message */
+	status = _gdata_service_send_message (self, message, error);
+	g_object_unref (message);
+	if (status == SOUP_STATUS_NONE)
+		return FALSE;
+
+	/* Check for cancellation */
+	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE)
+		return FALSE;
+
+	if (status != 200) {
+		/* Error */
+		/* TODO: Handle errors more specifically, making sure to set authenticated where appropriate */
+		g_set_error (error, GDATA_SERVICE_ERROR, GDATA_SERVICE_ERROR_WITH_DELETION,
+			     _("TODO: error code %u when deleting"), status);
 		return FALSE;
 	}
 
-	g_assert (message->response_body->data != NULL);
-	/* TODO: Update the entry such that it looks like it's been inserted */
-
 	return TRUE;
 }
 
+static void
+notify_proxy_uri_cb (GObject *gobject, GParamSpec *pspec, GObject *self)
+{
+	g_object_notify (self, "proxy-uri");
+}
+
+/**
+ * gdata_service_get_proxy_uri:
+ * @self: a #GDataService
+ *
+ * Gets the proxy URI on the #GDataService's #SoupSession.
+ *
+ * Return value: the proxy URI, or %NULL
+ *
+ * Since: 0.2.0
+ **/
+SoupURI *
+gdata_service_get_proxy_uri (GDataService *self)
+{
+	SoupURI *proxy_uri;
+
+	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
+
+	g_object_get (self->priv->session, SOUP_SESSION_PROXY_URI, &proxy_uri, NULL);
+	g_object_unref (proxy_uri); /* remove the ref added by g_object_get */
+
+	return proxy_uri;
+}
+
+/**
+ * gdata_service_set_proxy_uri:
+ * @self: a #GDataService
+ * @proxy_uri: the proxy URI
+ *
+ * Sets the proxy URI on the #SoupSession used internally by the given #GDataService.
+ * This forces all requests through the given proxy.
+ *
+ * Since: 0.2.0
+ **/
+void
+gdata_service_set_proxy_uri (GDataService *self, SoupURI *proxy_uri)
+{
+	g_return_if_fail (GDATA_IS_SERVICE (self));
+	g_object_set (self->priv->session, SOUP_SESSION_PROXY_URI, proxy_uri, NULL);
+	g_object_notify (G_OBJECT (self), "proxy-uri");
+}
+
 /**
  * gdata_service_is_authenticated:
  * @self: a #GDataService
@@ -1016,18 +1323,3 @@ gdata_service_get_password (GDataService *self)
 	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
 	return self->priv->password;
 }
-
-/**
- * gdata_service_get_session:
- * @self: a #GDataService
- *
- * Returns the #SoupSession which @self is using to communicate with the online service.
- *
- * Return value: a #SoupSession
- **/
-SoupSession *
-gdata_service_get_session (GDataService *self)
-{
-	g_return_val_if_fail (GDATA_IS_SERVICE (self), NULL);
-	return self->priv->session;
-}
