@@ -37,6 +37,8 @@ G_BEGIN_DECLS
  * @GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED: An entry has already been inserted, and cannot be re-inserted
  * @GDATA_SERVICE_ERROR_WITH_INSERTION: TODO
  * @GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED: The user attempted to do something which required authentication, and they weren't authenticated
+ * @GDATA_SERVICE_ERROR_WITH_UPDATE: TODO
+ * @GDATA_SERVICE_ERROR_WITH_DELETION: TODO
  *
  * Error codes for #GDataService operations.
  **/
@@ -46,7 +48,9 @@ typedef enum {
 	GDATA_SERVICE_ERROR_WITH_QUERY,/* TODO: probably should die */
 	GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED,
 	GDATA_SERVICE_ERROR_WITH_INSERTION,/* TODO: probably should die */
-	GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED
+	GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED,
+	GDATA_SERVICE_ERROR_WITH_UPDATE,/* TODO: probably should die */
+	GDATA_SERVICE_ERROR_WITH_DELETION/* TODO: probably should die */
 } GDataServiceError;
 
 /**
@@ -75,20 +79,6 @@ typedef enum {
 	GDATA_AUTHENTICATION_ERROR_SERVICE_DISABLED
 } GDataAuthenticationError;
 
-/**
- * GDataEntryParserFunc:
- * @doc: the XML document
- * @node: the XML node for an element; the <literal>&lt;entry&gt;</literal> element
- * @error: a #GError, or %NULL
- *
- * Callback function for operations which parse feeds, which should produce a new #GDataEntry from the provided entry XML.
- *
- * It can return any error, but #GDataParserError errors are most likely and most expected.
- *
- * Return value: a new #GDataEntry, or %NULL
- **/
-typedef GDataEntry *(*GDataEntryParserFunc) (xmlDoc *doc, xmlNode *node, GError **error);
-
 /**
  * GDataQueryProgressCallback:
  * @entry: a new #GDataEntry
@@ -153,9 +143,9 @@ typedef struct {
 				      const gchar *response_body, gint length, GError **error);
 } GDataServiceClass;
 
-GType gdata_service_get_type (void);
-GQuark gdata_service_error_quark (void);
-GQuark gdata_authentication_error_quark (void);
+GType gdata_service_get_type (void) G_GNUC_CONST;
+GQuark gdata_service_error_quark (void) G_GNUC_CONST;
+GQuark gdata_authentication_error_quark (void) G_GNUC_CONST;
 
 gboolean gdata_service_authenticate (GDataService *self, const gchar *username, const gchar *password, GCancellable *cancellable, GError **error);
 void gdata_service_authenticate_async (GDataService *self, const gchar *username, const gchar *password,
@@ -164,23 +154,32 @@ gboolean gdata_service_authenticate_finish (GDataService *self, GAsyncResult *as
 
 #include <gdata/gdata-query.h>
 
-GDataFeed *gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *query, GDataEntryParserFunc parser_func,
+GDataFeed *gdata_service_query (GDataService *self, const gchar *feed_uri, GDataQuery *query, GType entry_type,
 				GCancellable *cancellable,
-				GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error);
-void gdata_service_query_async (GDataService *self, const gchar *feed_uri, GDataQuery *query, GDataEntryParserFunc parser_func,
+				GDataQueryProgressCallback progress_callback, gpointer progress_user_data, GError **error) G_GNUC_WARN_UNUSED_RESULT;
+void gdata_service_query_async (GDataService *self, const gchar *feed_uri, GDataQuery *query, GType entry_type,
 				GCancellable *cancellable,
 				GDataQueryProgressCallback progress_callback, gpointer progress_user_data,
 				GAsyncReadyCallback callback, gpointer user_data);
-GDataFeed *gdata_service_query_finish (GDataService *self, GAsyncResult *async_result, GError **error);
+GDataFeed *gdata_service_query_finish (GDataService *self, GAsyncResult *async_result, GError **error) G_GNUC_WARN_UNUSED_RESULT;
 
-gboolean gdata_service_insert_entry (GDataService *self, const gchar *upload_uri, GDataEntry *entry, GCancellable *cancellable, GError **error);
-/* TODO: async versions */
+GDataEntry *gdata_service_insert_entry (GDataService *self, const gchar *upload_uri, GDataEntry *entry,
+					GCancellable *cancellable, GError **error) G_GNUC_WARN_UNUSED_RESULT;
+/* TODO: async version */
+
+GDataEntry *gdata_service_update_entry (GDataService *self, GDataEntry *entry, GCancellable *cancellable, GError **error) G_GNUC_WARN_UNUSED_RESULT;
+/* TODO: async version */
+
+gboolean gdata_service_delete_entry (GDataService *self, GDataEntry *entry, GCancellable *cancellable, GError **error);
+/* TODO: async version */
+
+SoupURI *gdata_service_get_proxy_uri (GDataService *self);
+void gdata_service_set_proxy_uri (GDataService *self, SoupURI *proxy_uri);
 
 gboolean gdata_service_is_authenticated (GDataService *self);
 const gchar *gdata_service_get_client_id (GDataService *self);
 const gchar *gdata_service_get_username (GDataService *self);
 const gchar *gdata_service_get_password (GDataService *self);
-SoupSession *gdata_service_get_session (GDataService *self);
 
 G_END_DECLS
 
