@@ -24,9 +24,18 @@
 
 G_BEGIN_DECLS
 
-#define G_TYPE_TIME_VAL (g_time_val_get_type ())
-GType g_time_val_get_type (void);
+#define GDATA_TYPE_G_TIME_VAL (gdata_g_time_val_get_type ())
+GType gdata_g_time_val_get_type (void) G_GNUC_CONST;
 
+/**
+ * GDataColor:
+ * @red: red color intensity, from 0–255
+ * @green: green color intensity, from 0–255
+ * @blue: blue color intensity, from 0–255
+ *
+ * Describes a color, such as used in the Google Calendar interface to
+ * differentiate calendars.
+ **/
 typedef struct {
 	guint16 red;
 	guint16 green;
@@ -34,7 +43,7 @@ typedef struct {
 } GDataColor;
 
 #define GDATA_TYPE_COLOR (gdata_color_get_type ())
-GType gdata_color_get_type (void);
+GType gdata_color_get_type (void) G_GNUC_CONST;
 gboolean gdata_color_from_hexadecimal (const gchar *hexadecimal, GDataColor *color);
 
 G_END_DECLS
