@@ -258,17 +258,35 @@ parse_error_response (GDataService *self, guint status, const gchar *reason_phra
 		xmlChar *domain = NULL, *code = NULL, *location = NULL;
 		xmlNode *child_node = node->xmlChildrenNode;
 
+		if (node->type == XML_TEXT_NODE) {
+			/* Skip text nodes; they're all whitespace */
+			node = node->next;
+			continue;
+		}
+
 		/* Get the error data */
 		while (child_node != NULL) {
+			if (child_node->type == XML_TEXT_NODE) {
+				/* Skip text nodes; they're all whitespace */
+				child_node = child_node->next;
+				continue;
+			}
+
 			if (xmlStrcmp (child_node->name, (xmlChar*) "domain") == 0)
 				domain = xmlNodeListGetString (doc, child_node->xmlChildrenNode, TRUE);
 			else if (xmlStrcmp (child_node->name, (xmlChar*) "code") == 0)
 				code = xmlNodeListGetString (doc, child_node->xmlChildrenNode, TRUE);
 			else if (xmlStrcmp (child_node->name, (xmlChar*) "location") == 0)
 				location = xmlNodeListGetString (doc, child_node->xmlChildrenNode, TRUE);
-			else {
-				/* Unknown element */
-				gdata_parser_error_unhandled_element ((gchar*) child_node->ns->prefix, (gchar*) child_node->name, "error", error);
+			else if (xmlStrcmp (child_node->name, (xmlChar*) "internalReason") != 0) {
+				/* Unknown element (ignore internalReason) */
+				if (*error == NULL) {
+					gdata_parser_error_unhandled_element ((gchar*) child_node->ns->prefix, (gchar*) child_node->name,
+									      "error", error);
+				} else {
+					g_warning ("Unhandled <%s:%s> element as a child of <error>.", child_node->ns->prefix, child_node->name);
+				}
+
 				xmlFree (domain);
 				xmlFree (code);
 				xmlFree (location);
@@ -407,8 +425,7 @@ gdata_youtube_service_query_standard_feed (GDataYouTubeService *self, GDataYouTu
 {
 	/* TODO: Support the "time" parameter, as well as category- and region-specific feeds */
 	return gdata_service_query (GDATA_SERVICE (self), standard_feed_type_to_feed_uri (feed_type), query,
-				    (GDataEntryParserFunc) _gdata_youtube_video_new_from_xml_node, cancellable,
-				    progress_callback, progress_user_data, error);
+				    GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
 }
 
 /**
@@ -436,8 +453,7 @@ gdata_youtube_service_query_standard_feed_async (GDataYouTubeService *self, GDat
 						 GAsyncReadyCallback callback, gpointer user_data)
 {
 	gdata_service_query_async (GDATA_SERVICE (self), standard_feed_type_to_feed_uri (feed_type), query,
-				   (GDataEntryParserFunc) _gdata_youtube_video_new_from_xml_node, cancellable, progress_callback, progress_user_data,
-				   callback, user_data);
+				   GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
 }
 
 /**
@@ -462,8 +478,7 @@ gdata_youtube_service_query_videos (GDataYouTubeService *self, GDataQuery *query
 				    GError **error)
 {
 	return gdata_service_query (GDATA_SERVICE (self), "http://gdata.youtube.com/feeds/api/videos", query,
-				    (GDataEntryParserFunc) _gdata_youtube_video_new_from_xml_node, cancellable,
-				    progress_callback, progress_user_data, error);
+				    GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
 }
 
 /**
@@ -490,8 +505,7 @@ gdata_youtube_service_query_videos_async (GDataYouTubeService *self, GDataQuery
 					  GAsyncReadyCallback callback, gpointer user_data)
 {
 	gdata_service_query_async (GDATA_SERVICE (self), "http://gdata.youtube.com/feeds/api/videos", query,
-				   (GDataEntryParserFunc) _gdata_youtube_video_new_from_xml_node, cancellable, progress_callback, progress_user_data,
-				   callback, user_data);
+				   GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
 }
 
 /**
@@ -529,8 +543,7 @@ gdata_youtube_service_query_related (GDataYouTubeService *self, GDataYouTubeVide
 
 	/* Execute the query */
 	return gdata_service_query (GDATA_SERVICE (self), related_link->href, query,
-				    (GDataEntryParserFunc) _gdata_youtube_video_new_from_xml_node, cancellable,
-				    progress_callback, progress_user_data, error);
+				    GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, error);
 }
 
 /**
@@ -570,8 +583,7 @@ gdata_youtube_service_query_related_async (GDataYouTubeService *self, GDataYouTu
 	}
 
 	gdata_service_query_async (GDATA_SERVICE (self), related_link->href, query,
-				   (GDataEntryParserFunc) _gdata_youtube_video_new_from_xml_node, cancellable, progress_callback, progress_user_data,
-				   callback, user_data);
+				   GDATA_TYPE_YOUTUBE_VIDEO, cancellable, progress_callback, progress_user_data, callback, user_data);
 }
 
 /**
@@ -702,7 +714,11 @@ gdata_youtube_service_upload_video (GDataYouTubeService *self, GDataYouTubeVideo
 	soup_message_set_request (message, "multipart/related; boundary=" BOUNDARY_STRING, SOUP_MEMORY_TAKE, upload_data, content_length);
 
 	/* Send the message */
-	status = soup_session_send_message (gdata_service_get_session (GDATA_SERVICE (self)), message);
+	status = _gdata_service_send_message (GDATA_SERVICE (self), message, error);
+	if (status == SOUP_STATUS_NONE) {
+		g_object_unref (message);
+		return NULL;
+	}
 
 	/* Check for cancellation */
 	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
